@@ -11,6 +11,23 @@ from rime_sdk.protos.test_run_tracker_pb2_grpc import TestRunTrackerStub
 # Generic type representing a client stub for a gRPC server.
 C = TypeVar("C")
 
+# GrpcApiKeyAuth is an AuthMetadataPlugin to add api key in header on all calls
+class GrpcApiKeyAuth(grpc.AuthMetadataPlugin):
+    """Auth plugin to add 'rime-api-key' header to grpc calls."""
+
+    def __init__(self, api_key: str) -> None:
+        """Create an Auth plugin to add 'rime-api-key' header to grpc calls.
+
+        Args:
+            api_key: str
+                Valid RIME Api Key to add to requests
+        """
+        self._api_key = api_key
+
+    def __call__(self, context: Any, callback: Callable) -> None:
+        """Add rime-api-key to grpc metadata when called."""
+        callback((("rime-api-key", self._api_key),), None)
+
 
 class RIMEConnection(Generic[C]):
     """A connection to a backend client of type C."""
@@ -19,6 +36,7 @@ class RIMEConnection(Generic[C]):
         self,
         create_backend_fn: Callable[..., C],
         addr: str,
+        api_key: str,
         channel_timeout: float = 5.0,
     ) -> None:
         """Create a new connection for a RIME backend.
@@ -29,10 +47,13 @@ class RIMEConnection(Generic[C]):
                 this connection.
             addr: str
                 The address of the backend server to create a channel to.
+            api_key: str
+                Api Key to validate RIME grpc requests with.
             channel_timeout: float
                 The timeout in seconds for waiting for the given channel.
         """
         self._create_backend_fn = create_backend_fn
+        self._api_key = api_key
         self._addr = addr
         self._channel_timeout = channel_timeout
         self._channel: Optional[grpc.Channel] = None
@@ -73,7 +94,10 @@ class RIMEConnection(Generic[C]):
         """
         try:
             # create credentials
-            credentials = self._get_ssl_channel_credentials()
+            credentials = grpc.composite_channel_credentials(
+                self._get_ssl_channel_credentials(),
+                grpc.metadata_call_credentials(GrpcApiKeyAuth(self._api_key)),
+            )
             channel = grpc.secure_channel(addr, credentials)
             grpc.channel_ready_future(channel).result(timeout=timeout)
             return channel
@@ -88,17 +112,20 @@ class RIMEConnection(Generic[C]):
 class RIMEBackend:
     """An abstraction for connecting to RIME's backend services."""
 
-    def __init__(self, domain: str, channel_timeout: float = 5.0):
+    def __init__(self, domain: str, api_key: str = "", channel_timeout: float = 5.0):
         """Create a new RIME backend.
 
         Args:
             domain: str
                 The base domain/address of the RIME service.+
+            api_key: str
+                The api key providing authentication to RIME services
             channel_timeout: float
                 The amount of time in seconds to wait for channels to become ready
                 when opening connections to gRPC servers.
         """
         self._channel_timeout = channel_timeout
+        self._api_key = api_key
         domain_split = domain.split(".", 1)
         if domain_split[0][-4:] != "rime":
             raise ValueError("The configuration must be a valid rime webapp url")
@@ -139,6 +166,7 @@ class RIMEBackend:
         return RIMEConnection[ModelTestingStub](
             ModelTestingStub,
             self._model_testing_addr,
+            self._api_key,
             channel_timeout=self._channel_timeout,
         )
 
@@ -147,6 +175,7 @@ class RIMEBackend:
         return RIMEConnection[ResultsStoreStub](
             ResultsStoreStub,
             self._results_store_addr,
+            self._api_key,
             channel_timeout=self._channel_timeout,
         )
 
@@ -155,5 +184,6 @@ class RIMEBackend:
         return RIMEConnection[TestRunTrackerStub](
             TestRunTrackerStub,
             self._test_run_tracker_addr,
+            self._api_key,
             channel_timeout=self._channel_timeout,
         )
